 .u-section-1 {
  background-image: linear-gradient(0deg, rgba(0,0,0,0.4), rgba(0,0,0,0.4)), url("images/gd4f267f778b22183ca1ded4dfae871eac04faaac23286dc27d9c228034ff735042cebd167fb684c47ea3bef8803094f5683f08a3728911b4b191f82a7c12de99_1280.jpg");
}

.u-section-1 .u-sheet-1 {
  min-height: 800px;
}

.u-section-1 .u-text-1 {
  margin: 313px auto 0;
}

.u-section-1 .u-text-2 {
  width: 826px;
  margin: 26px auto 60px;
}

@media (max-width: 1199px) {
  .u-section-1 .u-sheet-1 {
    min-height: 720px;
  }
}

@media (max-width: 991px) {
  .u-section-1 .u-text-2 {
    width: 720px;
  }
}

@media (max-width: 767px) {
  .u-section-1 .u-sheet-1 {
    min-height: 685px;
  }

  .u-section-1 .u-text-2 {
    width: 540px;
  }
}

@media (max-width: 575px) {
  .u-section-1 .u-sheet-1 {
    min-height: 564px;
  }

  .u-section-1 .u-text-2 {
    width: 340px;
  }
}